package jrummikub.model;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import jrummikub.util.Pair;
import static jrummikub.model.StoneColor.*;

import org.junit.*;
import static org.junit.Assert.*;

public class StoneSetTest {

	// Is Valid-Test
	// valid
	public void assertSet(boolean valid, List<Stone> stones) {
		StoneSet set = new StoneSet(stones);
		assertTrue(valid == set.isValid());
	}

	@Test
	public void doubleJoker() {
		assertSet(true, Arrays.asList(new Stone(RED), new Stone(BLACK),
				new Stone(1, BLACK)));
	}

	@Test
	public void groups() {
		assertSet(true, Arrays.asList(new Stone(1, RED), new Stone(1, BLACK),
				new Stone(1, BLUE)));
		assertSet(true, Arrays.asList(new Stone(1, RED), new Stone(1, BLACK),
				new Stone(1, BLUE), new Stone(1, ORANGE)));
	}

	@Test
	public void runs() {
		assertSet(true, Arrays.asList(new Stone(1, RED), new Stone(2, RED),
				new Stone(3, RED)));
		assertSet(true, Arrays.asList(new Stone(4, BLUE), new Stone(5, BLUE),
				new Stone(6, BLUE)));
	}

	@Test
	public void singleJoker() {
		assertSet(true, Arrays.asList(new Stone(1, RED), new Stone(1, BLACK),
				new Stone(RED)));
		assertSet(true, Arrays.asList(new Stone(2, RED), new Stone(3, RED),
				new Stone(BLACK)));
	}

	// invalid

	@Test
	public void outOfBounds() {
		assertSet(false, Arrays.asList(new Stone(RED), new Stone(1, RED),
				new Stone(2, RED)));
		assertSet(false, Arrays.asList(new Stone(12, RED), new Stone(13, RED),
				new Stone(RED)));
		assertSet(false, Arrays.asList(new Stone(RED), new Stone(BLACK),
				new Stone(1, RED), new Stone(2, RED)));
	}
	@Test
	public void sameColor() {
		assertSet(false, Arrays.asList(new Stone(1, RED), new Stone(1, RED),
				new Stone(1, BLUE)));
		assertSet(false, Arrays.asList(new Stone(1, RED), new Stone(1, BLUE),
				new Stone(1, BLACK), new Stone(1, ORANGE), new Stone(RED)));
	}

	@Test
	public void incorrectOrder() {
		assertSet(false, Arrays.asList(new Stone(4, RED), new Stone(6, RED),
				new Stone(5, RED)));	
		assertSet(false, Arrays.asList(new Stone(4, RED), new Stone(6, RED),
						new Stone(RED)));
		assertSet(false, Arrays.asList(new Stone(4, RED), new Stone(RED),
				new Stone(5, RED)));
	}
	
	@Test
	public void otherInvalid() {
	
		assertSet(false, Arrays.asList(new Stone(4, RED), new Stone(5, RED),
			new Stone(7, RED)));
		assertSet(false, Arrays.asList(new Stone(4, RED), new Stone(5, BLUE),
				new Stone(6, RED)));
		assertSet(false, Arrays.asList(new Stone(4, RED), new Stone(5, RED)));
		
	}
	// invalid Split
	@Test(expected = AssertionError.class)
	public void testSplitInvalidLow() {
		StoneSet testSet = createTestSet();
		testSet.splitAt(0);

	}

	@Test(expected = AssertionError.class)
	public void testSplitInvalidHigh() {
		StoneSet testSet = createTestSet();
		testSet.splitAt(3);

	}

	// valid Split
	@Test
	public void testSplitValid() {
		StoneSet testSet = createTestSet();
		Pair<StoneSet, StoneSet> newSets = testSet.splitAt(1);
		// Sets have right size
		assertEquals(1, newSets.getFirst().size());
		assertEquals(2, newSets.getSecond().size());
		// Set have right Stones
		assertSame(testSet.get(0), newSets.getFirst().get(0));
		assertSame(testSet.get(1), newSets.getSecond().get(0));
		assertSame(testSet.get(2), newSets.getSecond().get(1));
	}

	private StoneSet createTestSet() {
		List<Stone> stones = new ArrayList<Stone>();
		stones.add(new Stone(1, BLUE, false));
		stones.add(new Stone(1, RED, false));
		stones.add(new Stone(1, BLACK, false));
		StoneSet testSet = new StoneSet(stones);
		return testSet;
	}

	// join
	@Test
	public void testJoin() {
		StoneSet testSet = createTestSet();
		StoneSet secondSet = new StoneSet(new Stone(2, BLUE, false));
		StoneSet joinedSet = testSet.join(secondSet);
		// Sets have right size
		assertEquals(4, joinedSet.size());
		// Set have right Stones
		assertSame(testSet.get(0), joinedSet.get(0));
		assertSame(testSet.get(1), joinedSet.get(1));
		assertSame(testSet.get(2), joinedSet.get(2));
		assertSame(secondSet.get(0), joinedSet.get(3));
	}

	// iterator
	@Test
	public void testIterator() {
		StoneSet testSet = createTestSet();
		int i = 0;

		for (Stone stone : testSet) {
			assertSame(stone, testSet.get(i));
			i++;
		}

		assertEquals(i, testSet.size());
	}
}
